/*************************************************************************
 * Travis Veldkamp 10/11/2007                                            *
 * ipacutil@zumbrovalley.net                                             *
 * Copyright (C) 2007  Travis Veldkamp                                   *
 *                                                                       *
 * VERY simple routine for parsing the command line                      *
 *                                                                       *
 * This is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * at your option) any later version.                                    *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with Foobar; if not, write to:                                  *
 *                   Free Software Foundation, Inc.                      *
 *                   59 Temple Place                                     *
 *                   Suite 330                                           *
 *                   Boston, MA  02111-1307                              *
 *                   USA                                                 *
 *************************************************************************/


#include "cmd_line.h"
#include "pac_prog.h"
#include <string.h>
#include "util.h"

/****************************************************
 * Do a simple-minded parse of the command line     *
 *                                                  *
 * Returns: 0 if completed w/o error, -1 on error   *
 ****************************************************/
int parse_cmdline(int argc, char **argv, struct cmdline_struct *settings) {

   int curarg, idx;

   /* Setup defaults... try to pick reasonable values */
   settings->showhelp = FALSE;
   settings->verbose = VERBOSE_NORMAL;
   settings->board_number= 0;             /* default to 1st board          */
   settings->run_detection = FALSE;       /* default is no board detection */
   settings->no_program = FALSE;          /* default is to try to program  */
   settings->ledbytes[0] = 0;
   settings->ledbytes[1] = 0;
   settings->get_current = 0;

   for (curarg=1; curarg<argc;curarg++) {
      if ((strcmp(argv[curarg], HELP_FLAG1) == 0) ||
          (strcmp(argv[curarg], HELP_FLAG2) == 0)) {

         settings->showhelp = 1;
         continue;
      }

      if ((strcmp(argv[curarg], VERBOSE_FLAG1) == 0) ||
          (strcmp(argv[curarg], VERBOSE_FLAG2) == 0)) {

         settings->verbose = VERBOSE_VERBOSE;
         set_debug_level(settings->verbose);
         continue;
      }

      if ((strcmp(argv[curarg], QUIET_FLAG1) == 0) ||
          (strcmp(argv[curarg], QUIET_FLAG2) == 0)) {
         settings->verbose = VERBOSE_QUIET;
         set_debug_level(settings->verbose);
         continue;
      }

      if ((strcmp(argv[curarg], DETECT_FLAG1) == 0) ||
          (strcmp(argv[curarg], DETECT_FLAG2) == 0)) {
         settings->run_detection = 1;
         continue;
      }

      if ((strcmp(argv[curarg], ALLON_FLAG1) == 0) ||
          (strcmp(argv[curarg], ALLON_FLAG2) == 0)) {

         settings->ledbytes[0] = LED_BYTE_ALL_ON;
         settings->ledbytes[1] = LED_BYTE_ALL_ON;
         continue;
      }

      if ((strcmp(argv[curarg], ALLOFF_FLAG1) == 0) ||
          (strcmp(argv[curarg], ALLOFF_FLAG2) == 0)) {

         settings->ledbytes[0] = LED_BYTE_ALL_OFF;
         settings->ledbytes[1] = LED_BYTE_ALL_OFF;
         continue;
      }

      if ((strcmp(argv[curarg], CURRENT_FLAG1) == 0) ||
          (strcmp(argv[curarg], CURRENT_FLAG2) == 0)) {

          settings->get_current = 1;
          continue;
      }


      if ((strcmp(argv[curarg], LED_FLAG1) == 0) ||
          (strcmp(argv[curarg], LED_FLAG2) == 0)) {
          printe("LED flag not handled yet\n");
          continue;
      }

      if ((strcmp(argv[curarg], STATE_FLAG1) == 0) ||
          (strcmp(argv[curarg], STATE_FLAG2) == 0)) {

         int  tval;

         /* The user is specifying all LED states via
          * a 16-bit hexadecimal number: ie 0xffff 
          */ 
         curarg++;
         
         if (argc <= curarg) {
             printe("No LED state specified for %s\n", argv[curarg-1]);
             return -1;
         } else {
             if ((strncasecmp(argv[curarg], "0x", 2) == 0) &&
                 (strlen(argv[curarg]) == 6)) {

                     // format 0xffff
                     tval = strtol(argv[curarg], NULL, 16);
                     settings->ledbytes[1] = tval & 0xff;
                     settings->ledbytes[0] = (tval >> 8) & 0xff;
             } else {
                 printe("LED state (%s) not specified correctly.  Expect form: 0xffff\n", argv[curarg]);
                 return -1;
             }
         }
         continue;
      }

      char matched=0;
      for(idx = 0; idx < 2; idx++) {
          char *tflag;
          char tstr[255], mask, *byteptr;
          int len, lednum, ledval;

          if (idx==0) {
              tflag = (char *)LED_FLAG1;
          } else {
              tflag = (char *)LED_FLAG2;
          }

          len = strlen(tflag);
          if (strncmp(argv[curarg], tflag, len) == 0) {
              /* Looks like user is specifying an LED to turn on/off */
              strncpy(tstr, &argv[curarg][len], 254);
              tstr[254] = '\0';

              curarg++;
              if (argc <= curarg) {
                  printe("No LED state specified for %s\n", argv[curarg-1]);
                  return -1;
              }

              ledval = atoi(argv[curarg]);
              if ((ledval < 0) || (ledval > 1)) {
                  printe("Invalid LED state specified for %s.  Should be 0 (off) or 1 (on)\n", argv[curarg-1]);
                  return -1;
              }

              lednum = atoi(tstr);
              if ((lednum < BYTE1_LED_INDEX_START) ||
                  (lednum > BYTE2_LED_INDEX_END)) {
                  printe("Invalid LED index.  Should be in range %d to %d\n", LED_INDEX_START, LED_INDEX_END);
                  return -1;
              }

              if (lednum < BYTE2_LED_INDEX_START) {
                  mask = 1 << (lednum - BYTE1_LED_INDEX_START);
                  byteptr = &(settings->ledbytes[1]);
              } else {
                  byteptr = &(settings->ledbytes[0]);
                  mask = 1 << (lednum - BYTE2_LED_INDEX_START);
              }

              if (ledval) {
                  *byteptr = *byteptr | mask;
              } else { 
                  *byteptr = *byteptr & ~mask;
              }
            
              matched=1;

          }  // if flag matches

      } //for
      if (matched) continue;

      if ((strcmp(argv[curarg], NO_PGM_FLAG1) == 0) ||
          (strcmp(argv[curarg], NO_PGM_FLAG2) == 0)) {

         /*User is specifying no programming cycle*/
         /*This is probably only useful when combined*/
         /*with the detect flag*/
         settings->no_program = 1;
         continue;
      }

      if ((strcmp(argv[curarg], BOARD_FLAG1) == 0) ||
          (strcmp(argv[curarg], BOARD_FLAG2) == 0)) {

         /*The user is specifying which board to use*/
         curarg++;

         if (argc <= curarg) {
             printf("No board specified for %s\n", argv[curarg-1]);
             return -1;
         } else {
             settings->board_number = atoi(argv[curarg]);
             if ((settings->board_number < 0) || (settings->board_number > MAX_PACS)) {
                 printf("**Invalid board specified: %s. **\n",
                        argv[curarg]);
                 return -1;
             }
         }

         continue;
      }


      /*If we get to this point, we didn't match the argument with anything*/
      printe("**Error: unknown command line argument found: %s**\n", argv[curarg]);
      return -1;
   }
   return 0;
}

void cmdline_ShowUsage(void) {
   printf("Usage: pacdrive [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s hexnum] [%s hexnum] [%sn 0/1] [%sn 0/1] [%s num] [%s num]\n",
          HELP_FLAG1, HELP_FLAG2, VERBOSE_FLAG1, VERBOSE_FLAG2, QUIET_FLAG1, QUIET_FLAG2,
          DETECT_FLAG1, DETECT_FLAG2, 
          ALLON_FLAG1, ALLON_FLAG2, ALLOFF_FLAG1, ALLOFF_FLAG2, STATE_FLAG1, STATE_FLAG2,
          LED_FLAG1, LED_FLAG2, BOARD_FLAG1, BOARD_FLAG2
           );
   printf("%s | %s\t\t-- print this information\n", HELP_FLAG1, HELP_FLAG2);
   printf("%s | %s\t\t-- turn on verbose mode\n", VERBOSE_FLAG1, VERBOSE_FLAG2);
   printf("%s | %s\t\t-- don't print anything to screen except errors\n", QUIET_FLAG1, QUIET_FLAG2);
   printf("%s | %s\t\t-- detect connected I-PACs\n", DETECT_FLAG1, DETECT_FLAG2);
   printf("%s | %s\t\t\t-- Turn all LEDs on\n", ALLON_FLAG1, ALLON_FLAG1);
   printf("%s | %s\t\t\t-- Turn all LEDs off\n", ALLOFF_FLAG1, ALLOFF_FLAG1);
   printf("%s | %s\t\t-- Set LEDs to specified state (16-bit bitmask such as 0xffff for all on)\n", STATE_FLAG1, STATE_FLAG2);
   printf("%s | %s\t\t-- Set LED n on (1) or off (0).  Use this flag multiple times to specify multiple LEDs\n", LED_FLAG1, LED_FLAG2);
   printf("%s | %s\t\t-- Specify the board number (from detection) to use\n", BOARD_FLAG1, BOARD_FLAG2);
   printf("\nThis is free software; see the source for copying conditions.  There is NO\n");
   printf("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");

}
