/*************************************************************************
 * TLV 10/11/2007                                                        *
 * Copyright (C) 2007 Travis Veldkamp                                    *
 *                                                                       *
 * This program is for programming the I-PAC keyboard encoder            *
 * For more information, see http://www.ultimarc.com                     *
 *                                                                       *
 * This is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * at your option) any later version.                                    *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with Foobar; if not, write to:                                  *
 *                   Free Software Foundation, Inc.                      *
 *                   59 Temple Place                                     *
 *                   Suite 330                                           *
 *                   Boston, MA  02111-1307                              *
 *                   USA                                                 *
 *************************************************************************/


#include <errno.h>
#include <fcntl.h>
#include <hid.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <usb.h>

#define PAC_USB_TIMEOUT	100

#define VENDOR_ID_ULTIMARC       0xD209
#define PRODUCT_ID_IPAC_USB      0x0301 
#define PRODUCT_ID_PACDRIVE      0x1500

#define PAC_DATA_HEADER_LEN 2
#define PAC_DATA_DATA_LEN   2

typedef struct _PAC_DATA_BLOCK {
    char Header[PAC_DATA_HEADER_LEN];
    char Data[PAC_DATA_DATA_LEN];
} PacData;


/* used by libhid to send/recv reports (input/output reports)*/
/* very simple paths, thankfully (from lsusb -vvv) */
#define PAC_PATHLEN 1
static int const PAC_PATH_IN[PAC_PATHLEN] =  { 0x00090001 };
static int const PAC_PATH_OUT[PAC_PATHLEN] = { 0x0008004b };

/* I can't imagine it would ever be more than this... so just use an array instead of llist */
/* CURRENTLY, only support 1 board at a time... TODO: investigate what it will take to support multiples */
//#define MAX_PACS	5
#define MAX_PACS	1

/* Populates hid_arr with HIDs which are Pac-Drives.  Returns number found */
int find_pacs(int max_num);

/* Set the LED states */
int set_led_state(HIDInterface *pac, PacData *data);

/* Get the current LED state */
int get_led_state(HIDInterface *pac, PacData *data);

/* Called on each item of hid_arr before used for get/set_led_state */
/* TODO: init is now done in find_pacs... either move it back here or get rid of this */
HIDInterface *init_pac(int board_num);

/* Called when completely done talking to pac -- the is the opposite of init_pac*/
void cleanup_pac(void);

/* Close the HID after done doing get/set */
int close_pac(HIDInterface *pac);

/* Get the number of Pac-Drives found attached */
int pac_count(int interface);
