/*************************************************************************
 * TLV 10/11/2007                                                        *
 *                                                                       *
 *************************************************************************/

#include "util.h"

static int util_current_verbosity = VERBOSE_NORMAL;
static FILE *util_log_hdl;

void set_debug_level(int level) {
    util_current_verbosity = level;
}


/**********************************************
 * Print verbose - take a verbosity level and *
 * print if we exceed some specified limit    *
 * The code for this routine is based on:     *
 * http://www.eskimo.com/~scs/C-faq/q15-5.html*
 **********************************************/
void printv(int trigger, char *fmt, ...) {

     va_list argp;

     /*Currently, there are only 2 settings - verbose or not verbose*/
     if (util_current_verbosity >= trigger) {
        va_start(argp, fmt);
        vprintf(fmt, argp);
        va_end(argp);
     }

}

/**********************************************
 * Print to log -  take a verbosity level and *
 * print if we exceed some specified limit    *
 * The code for this routine is based on:     *
 * http://www.eskimo.com/~scs/C-faq/q15-5.html*
 **********************************************/
void printl(int trigger, char *fmt, ...) {
     FILE *loghdl = util_log_hdl;
     va_list argp;

     if (loghdl == NULL) {
         util_log_hdl = stdout;
     }

     /*Currently, there are only 2 settings - verbose or not verbose*/
     if (util_current_verbosity >= trigger) {
        va_start(argp, fmt);
        vfprintf(loghdl, fmt, argp);
        va_end(argp);
     }
}

/**********************************************
 * Print error - print an error message in a  *
 * standard format                            *
 * The code for this routine is from:         *
 * http://www.eskimo.com/~scs/C-faq/q15-5.html*
 **********************************************/
void printe(char *fmt, ...) {

   va_list argp;
   fprintf(stderr, "Error: ");
   va_start(argp, fmt);
   vfprintf(stderr, fmt, argp);
   va_end(argp);
}

void openlog(char *logfile) {
   if (util_log_hdl != NULL) {
      closelog();
   }

   util_log_hdl = fopen(logfile, "w");
   if (util_log_hdl == NULL) {
       util_log_hdl = stdout;
       printe("Failed to open log file.  Dumping garbage to screen\n");
   };

}

void closelog(void) {
   if ((util_log_hdl != NULL) && (util_log_hdl != stdout)) {
       fclose(util_log_hdl);
   }
   util_log_hdl = NULL;
}

void clean_exit(int code) {
   closelog();
   exit(code);
}
